#include "hardware.h"
#include <avrx.inc>

;+
;-------------------------------------------------------
;
; ReadProximitySensor
;
; PASSED:       Nothing
; RETURNS       Nothing
; USES:         R0, R20-31, flags
; STACK:        2
; NOTES:        Uses a semaphore to control access to the sensors.
;               Usage: Call Init_GP2D02() during startup.  Then just
;               call Read_GP2D02() to take a sample.  Process hangs for
;               up to 62ms. 
;-
        .comm   FrontRaw, 1
        .comm   RightRaw, 1
        .comm   LeftRaw, 1
	.lcomm	GP2D02_Timer, TcbSz
	.lcomm  GP2D02_Flag, MtxSz

        .func   Read_GP2D02
        .global Read_GP2D02
Read_GP2D02:
        ldi     R24, lo8(GP2D02_Flag)
        ldi     R25, hi8(GP2D02_Flag)
        rcall   AvrXWaitSemaphore
        
	ldi	R24, lo8(GP2D02_Timer)
	ldi	R25, hi8(GP2D02_Timer)
        rcall   AvrXWaitTimer
        
        cbi     DIN                     ; Initiate Proximity Detection

	ldi	R24, lo8(GP2D02_Timer)
	ldi	R25, hi8(GP2D02_Timer)
	ldi	R22, lo8(60*TICKRATE/1000)
	ldi	R23, hi8(60*TICKRATE/1000)
	rcall	AvrXDelay
        
        ldi     Zh, 9                   ; Use bit counter so having no
                                        ; sensor wired up does not hang routine
ReadNextBit:
        in      R0, SHARP-2             ; Read pins, not pullups
        sbi     DIN

        ror     R0                      ; First bit is Vin
        ror     R0                      ; Bits 0-2 are Front, Right, Left
        rol     R20                     ; R20 = Front
        ror     R0
        rol     R21                     ; R21 = Right
        ror     R0
        rol     R22                     ; R22 = Left
        
        ldi     Zl,(1*CPUCLK)/3000000   ; 1 uS delay
1:
        dec     Zl
        brne    1b
        cbi     DIN
        ldi     Zl, (16*CPUCLK)/3000000 ; 16 uS delay
2:
        dec     Zl
        brne    2b
        dec     Zh
        brne    ReadNextBit
        ;
        ; Cook and Store results
        ;
        sts     FrontRaw, R20
        sts     RightRaw, R21
        sts     LeftRaw, R22
        ;
;        rjmp    Init_GP2D02            ; Fall through
        
	.endfunc
;
; Called ONCE prior to using the read routine, above
;
        .global Init_GP2D02
        .func   Init_GP2D02
Init_GP2D02:
        sbi     DIN
	ldi	R24, lo8(GP2D02_Timer)
	ldi	R25, hi8(GP2D02_Timer)
	ldi	R22, lo8(2*TICKRATE/1000)
	ldi	R23, hi8(2*TICKRATE/1000)
	rcall	AvrXStartTimer
        ldi     R24, lo8(GP2D02_Flag)
        ldi     R25, hi8(GP2D02_Flag)
        rjmp    AvrXSetSemaphore

        .endfunc

    
