#include <io-avr.h>
#include <signal.h>
#include "myctoasm.inc"
#define BV(A) (1<<A)
/*
 interrupt driven ADC conversion drivers.

 Does continuous conversions of all 8 channels, starting with #7, then
 decrementing to 0 and rolling over to 7 again.

 53 cycles/interrupt.  At 8mhz it is roughly 432/900*8 or ~6% load.

 Uses a 1/64 clock.  On an 8mhz machine each conversion takes roughly
 100us for a total conversion time of ~.9ms or roughly 8% load.

 */

        .lcomm  adc_channel, 1  ; Internal: current channel number
;        .global adc
        .lcomm  adc, 16         ; Results

        .section .text.adcdriver
        .func   SIG_ADC
        .global SIG_ADC
        
SIG_ADC:
        push    R28
        in      R28, SREG
        push    R28
        push    R30
        push    R31

        ldi     R30, lo8(adc)
        ldi     R31, hi8(adc)
        lds     R28, adc_channel

        add     R28, R28
        add     R30, R28
        sbci    R31, lo8(-0)    ; Add channel # to index
        in      R28, ADCL
        st      Z+, R28         ; store ADC results little endien for C
        in      R28, ADCH
        st      Z, R28

        lds     R28, adc_channel
        subi    R28, 1          ; If < 0, then wrap to 7
        brcc    continue
        ldi     R28, lo8(7)
continue:
        out     ADMUX, R28
        sts     adc_channel, R28
        sbi     ADCSR, ADSC     ; Get next channel results
        pop     R31
        pop     R30
        pop     R28
        out     SREG, R28
        pop     R28
        reti
        .endfunc

;
; void ADC_StartConversion(void)
;
; Initializes Channel to 7, then starts a conversion
; Enable the interrupt flag (never disabled afterwards)
;
        .func   ADC_Init
        .global ADC_Init
        
ADC_Init:
        ldi     R24, BV(ADEN) | BV(ADSC) | BV(ADPS1) | BV(ADPS2)
        out     ADCSR, R24
1:
        sbic    ADCSR, ADSC     ; Wait until dummy conversion done.
        rjmp    1b
        
        ldi     R24, lo8(7)
        sts     adc_channel, R24
        out     ADMUX, R24
        sbi     ADCSR, ADSC
        sbi     ADCSR, ADIE
        ret
        .endfunc

/*
 unsigned short ADC_Channel(unsigned char)

 Atomic read of a word from the ADC data array
*/
        .func   ADC_Channel
        .global ADC_Channel
ADC_Channel:
        ldi     R30, lo8(adc)
        ldi     R31, hi8(adc)
        add     p1b, p1b
        add     R30, p1b
        sbci    R31, -(0)
        in      R0, SREG
        cli
        ld      R24, Z+
        ld      R25, Z
        out     SREG, R0
        ret
        .endfunc
