#ifndef CHEADER_HARDWARE
# define CHEADER_HARDWARE

/*
File:   HARDWARE.INC

Copyright 2000, 2001 Larry Barello

*/

// Here is the *real* hardware!

#define DEG_PER_TICK    (DIST_PER_TICK/WHEEL_BASE)
#define ONE_DEG         ((int)((2*M_PI*256)/(DEG_PER_TICK*360)))            // Ticks/degree
#define WHEEL_DIAMETER  1.99295
#define WHEEL_BASE      5.03
#define DIST_PER_TICK   (M_PI*WHEEL_DIAMETER/800) // Inches per tick

#define LEFT_WHEEL      2.0016                                            // Each wheel is slightly different
#define RIGHT_WHEEL     1.9843
#define DISTANCE(A)     (M_PI*(A)/800)
#define DEGREE(A)       ((int)((2*M_PI*256)/(DISTANT(A) * 360 / WHEELBASE)))

// Here is the CPU stuff

#  define CPUCLK       9683040        // Xtal frequency
//#define CPUCLK      11059200
//#define CPUCLK       4000000        // Test Board Xtal
#define BAUDRATE 19200
#define UBRR_INIT ((CPUCLK/16/BAUDRATE)-1)
//
// System Clock settings: CPU/256, 3khz tickrate
//
#define TICKRATE     1500           // Picked to give an even divisor of CPU clock
#define MILLISECOND(A) ((A)*(long)TICKRATE/1000)
#define HERTZ(A)    (TICKRATE/(A))
#define TCCR0_INIT  (1<<CS02)
#define TCNT0_INIT  (256-(CPUCLK/256/TICKRATE))
//
// MCUCR = Int0/1 negative edge triggered, Sleep enabled
// TIMSK = Enable timer0 interrupt
// GIMSK = Enable int0/1 interrupts
//
#define MCUCR_INIT ((1<<SE) | (1<<ISC11) | (1<<ISC01))
#define TIMSK_INIT (1<<TOIE0)
#define GIMSK_INIT ((1<<INT1) | (1<<INT0))
//
// High Speed Counter settings:  CPU/1, 8 bit PWM 
//
#define TCCR1A_INIT  ((1<<PWM10) | (1<<COM1A1) | (1<<COM1B1))
#define TCCR1B_INIT  (1<<CS10)
//
// Right Motor
// PD5 = OC1A
//
#define RIGHT_DRIVEH OCR1BH
#define RIGHT_DRIVEL OCR1BL
//
// Left Motor
// PD4 (OC1B)
//
#define LEFT_DRIVEH  OCR1AH
#define LEFT_DRIVEL  OCR1AL
//
// PORTB Bit Definitions
//
// PB 1-3 are the left, Front, Right Sharp Proximity Sensors
// PB0 is the control signal, initialized to 1 to reset sensors
//
#define Vin     PB0         // Control signal into Sharp Proximity Sensor
#define DORIGHT PB1         // Data Out from Sharp Proximity Sensors
#define DOCENTER PB2
#define DOLEFT  PB3
#define FAN     PB4         // SS
#define LED     PB5         // MOSI
#define SW      PB7
#define DIN     PORTB, Vin          // Control signal to Sharp Sensors
#define MODE_SW PINB, SW
//
#define GP2D02_MASK     (1<<DOLEFT | 1<<DORIGHT | 1<<DOCENTER)
#define PORTB_INIT      (1<<Vin  | 1<<SW | GP2D02_MASK)  // 1 resets sensors, need pullups for not present sensors
#define DDRB_INIT       ((1<<LED) | (1<<FAN) | (1<<Vin)) // Vin, Fan and LED are outputs
#define SHARP           PORTB
//
// Port D Bit Definitions
//
// PD 0,1 used for Serial I/O
// PD 4,5 used for PWM output
//
#define Right_Enc_A     PIND, PD2   // INT0 
#define Right_Enc_B     PIND, PD6
#define Left_Enc_A      PIND, PD3   // INT1
#define Left_Enc_B      PIND, PD7

#define ENCODER       800           // Number of clicks/revolution

#define DDRD_INIT     1<<PD5 | 1<<PD4 | 1<<PD1        // PWM and Txd are outputs
#define PORTD_INIT    ~DDRD_INIT    // Enable pullups on all inputs
//
// PORT A Bit Definitions
//
#define PORTA_INIT   0              // ADC port
#define DDRA_INIT    0
//
//
// PORT C Bit Definitions (LCD Port & Microphone input)
//
#define PORTC_INIT      0x00        // Upper nibble is data buss
#define DDRC_INIT       0x0E        // PC1-3 are control lines.
#define MIC             PC0
#define MICROPHONE      PINC, MIC   // Use PC0 for microphone input

//------------------------------------------------------
// Data Structures and various constants
//------------------------------------------------------

#define MIC_THRESHOLD 0x03            // Debounce x100ms per count

//
// Mode Switch parameters (in 1/10 seconds)
//
#define TAP           1
#define LONG          5
#define RESET         15
//
//
// Heuristic values for our four sensors.  The output
// is =XXXX_FG/(SensorOutput-XXXX_FO)
//
// These values calibrate the sensor cooked output in 1/10's
// of an inch
//
#define FRONT_KG      0x1DB0
#define FRONT_KO      0x11
#define RIGHT_KG      0x1CA5
#define RIGHT_KO      0x25
#define REAR_KG       0x1932
#define REAR_KO       0x11
#define LEFT_KG       0x1B9B
#define LEFT_KO       0x15

#endif // CHEADER_HARDWARE

