/*
lcd formatted i/o and access routines

Copyright 1998, 1999 Larry Barello

Author: Larry Barello
        larry@barello.net
*/
#include "AvrX.inc"
#include "lcd.inc"
#include "myctoasm.inc"
;+
; Change History
;
; 4-13-00       Lcd_EraseToEndOfLine
;               What was I thinking?  The rjmp after the call
;               to print out the space was "brne"?  Duh?  There
;               is no return value from Lcd_WriteCommand!
;-
        .global lcdmutex
        .lcomm  lcdmutex, MtxSz
;+
;------------------------------------------------------------------
; lcd_Get/ReleaseDisplay
;
; Uses lcdmutex to control access to the LCD driver
;
; PASSED:       Nothing
; RETURNS:      Nothing
; USES:         Nothing
; NOTES:        Calling process may block waiting for access
;               to the LCD display.  If you call this twice
;               you will DEADLOCK
;-
        
        .global lcd_GetDisplay
        .func   lcd_GetDisplay
lcd_GetDisplay:
        ldi     p1wl, lo8(lcdmutex)
        ldi     p1wl, hi8(lcdmutex)
        rjmp    AvrXWaitSemaphore
        .endfunc

        .global lcd_ReleaseDisplay
        .func   lcd_ReleaseDisplay
lcd_ReleaseDisplay:
        ldi     p1wl, lo8(lcdmutex)
        ldi     p1wl, hi8(lcdmutex)
        rjmp    AvrXSetSemaphore
        .endfunc

;-----------------------------------------
;void lcd_PutHex(uint16_t, uint8_t)
; 
; Passed:       p1wh:p1wl = Value, p2b = # digits
; RETURN:       Nothing
; USES:         X, R22
; STACK:        2
;
        .global lcd_PutHex
        .func   lcd_PutHex
lcd_PutHex:
        mov     Xl, p1wl
        mov     Xh, p1wh
        ldi     p3b, 4          ; Maximum number of digits.
1:
        lsl     Xl
        rol     Xh
        rol     p1b
        lsl     Xl
        rol     Xh
        rol     p1b
        lsl     Xl
        rol     Xh
        rol     p1b
        lsl     Xl
        rol     Xh
        rol     p1b

        cp      p2b, p3b
        brlt    3f

        cbr     p1b, 0xF0
        subi    p1b, -'0'          ; Addi 0
        cpi     p1b, ('9' + 1)     ; if > 9
        brlt    2f                 ; subi 9 and add A
        subi    p1b, ('9' + 1 - 'A')
2:
        rcall   lcd_wr_data     ; Uses p1w{h,l}
3:  
        dec     p3b
        brne    1b

        ret
        .endfunc
        
;-----------------------------------------
; void lcd_PutHexWord(uint16_t)
; 
; Passed: p1wh:p1wl = word
;
        .global lcd_PutHexWord
        .func   lcd_PutHexWord
lcd_PutHexWord:
        ldi     p2b, 4
        rjmp    lcd_PutHex
        .endfunc

;-----------------------------------------
; void lcd_PutHexByte(uint8_t)
; 
; Passed: R25 = Byte
;
        .global lcd_PutHexByte
        .func   lcd_PutHexByte
lcd_PutHexByte:
        ldi     p2b, 2
        rjmp    lcd_PutHex
        .endfunc

        .global lcd_PutHexChar
        .func   lcd_PutHexChar
lcd_PutHexChar:
        ldi     p2b, 1
        rjmp    lcd_PutHex
        .endfunc
        

;
;--------------------------------------------------
;
; void lcd_putstr(pFlash)
;
; Puts out an ascii string to the LCD display
;
; PASSED:       Z = pointer to string in CODE
; RETURNS:
; USES:         LCD_PARM, R0
; CALLS:        lcd_wr_data
; NOTES:        
;
        .global lcd_putstr
        .func   lcd_putstr
lcd_putstr:
        mov     Zl, p1wl
        mov     Zh, p1wh
1:
        lpm
        mov     p1b, R0
        tst     p1b
        breq    ps_x
        rcall   lcd_wr_data
        adiw    Zl, 1
        rjmp    1b
ps_x:
        ret
        .endfunc

;+
; void lcd_PrintMessageAt(pFlash, Cursor)
;
; Print a FLASH string at a location set by LCD_PARM
; Erase all charactors after the string, to the end of line
;
; Passed:       p1wl:p1wl = Pointer to Flash String
;               p2b = LCD cursor location
; Returns:      nothing
; Uses:         Z
;-
        .global lcd_PrintMessageAt
        .func   lcd_PrintMessageAt
lcd_PrintMessageAt:
        push    p1wl
        push    p1wh
        ldi     p1b, LCD_SET_DDRAM
        or      p1b, p2b
        rcall   lcd_wr_inst
        pop     p1wh
        pop     p1wl
        rcall   lcd_putstr
        rcall   lcd_EraseToEndOfLine
        ret
        .endfunc
;+
; void lcd_EraseToEndOfLine(void)
;
; Erases the LCD from the current cursor position to
; the end of the line.
;
; Passed:       Nothing
; Returns:      Nothing
; Uses:         LCD_PARAM
; NOTE:         Writes " " before checking so it
;               will clear from start of line.
;-
        .global lcd_EraseToEndOfLine
        .func   lcd_EraseToEndOfLine
lcd_EraseToEndOfLine:
        ldi     p1b, 0x20
        rcall   lcd_wr_data
        
        rcall   lcd_rd_inst     ; Clear to end of line.
        cbr     p1b, 0x80
        cpi     p1b, LCD_LINE1
        breq    EraseDone
        cpi     p1b, LCD_LINE2
        breq    EraseDone
        cpi     p1b, LCD_LINE3
        breq    EraseDone
        cpi     p1b, LCD_LINE4
        brne    lcd_EraseToEndOfLine
        
EraseDone:
        ret
        .endfunc
        
