// Error codes for halting.

#define INVALID_BRANCH0 0
#define INVALID_BRANCH1 1
#define INVALID_BRANCH2 2
#define INVALID_FOLLOWLINE 3
#define STACK_FULL 4
#define TIMEOUT_DEADEND 5

typedef union
{
    struct
    {
        unsigned left_present:1;
        unsigned left_visited:1;
        unsigned right_present:1;
        unsigned right_visited:1;
        unsigned center_present:1;
        unsigned center_visited:1;
        unsigned branch:2;
    } b;
    unsigned char byte;
} NodeFlags;

typedef struct
{
    NodeFlags flags;
    char iX;        // Inches
    char iY;
    char iTheta;    // Degrees
} Node;

// Return values for FollowLine()

#define DEAD_END 0          // 1-3 Indicate # branches
#define GOAL    4
#define TIMEOUT 5           // 3 minute time out

#define MAZE_TIMEOUT 1780    // 2:58 minute timeout @ 10hz

// # of degrees to turn for each value

#define RIGHT   (-100)      // Used in Turn()
#define LEFT    (100)
#define BACK    (200)
#define TO_CENTER_LINE 1    // Turn and seek center line

#define MAXNODES 40         // fourty true intersections... is a lot.

#define LEFT_BRANCH 0       // Used in "Current Branch" field
#define RIGHT_BRANCH 1
#define CENTER_BRANCH 2

// Line Follow Parameters

#define LEFTBRANCH  5       // ADC Channels
#define LEFTLINE    2
#define CENTERLINE  3
#define RIGHTLINE   4
#define RIGHTBRANCH 6

#define FloorScale              7
#define FollowVelocity          10 << 8 // 8.8 number
#define FloorThresholdLow       0xF0
#define FloorThresholdHigh      0x1B0

// Loop Counter = 1/2" / (DIST_TICK * FollowVelocity)

#define FollowLineHalfInch      6       // Loops at follow velocity

// Counter = 3.5" / (DIST_TICK * FollowVelocity) - Deceleration (11)
// Deceleration is the integral of FollowVelocity - acceleration, or $A00-$70
// Pretty much have to calculate this by hand...

#define FollowLineThreeHalfInch 33  // "" adjusted for deceleration
#define FollowHallSixInches 65      // = 6/.078 - 11 (

typedef struct
{
    int iFloorScale;
    int iFollowVelocity;
    int iThresholdLow;
    int iThresholdHigh;
}
FloorSensorParameters;


int GetLineTrackInfo(int);
void Turn(int, char);
int PushNode(Node *, NodeFlags);
void DoFloorThreshold(void);
void InitFloorParameters(void);
void WritepNode(Node *);
void WriteNode(Node *, NodeFlags);
int SynchWithMotion(void);
void WaitForMotionToStop(void);

extern Node eeNodeData[] EEPROM;
extern int EEPROM eepNode;
extern NodeFlags Sensor;
extern FloorSensorParameters FloorData;
extern int MazeOption;
extern int MazeTimer;

