/*
 Monitor.c
*/
#include <io.h>
#include <AvrX.h>
#include <serialio.h>
#include "MotorTask.h"
#include <ctype.h>
#include <eeprom.h>
#include <adc_driver.h>
#include <maze.h>

#define FALSE 0
#define TRUE 1

extern uint16_t EEPROM Acceleration;

uint16_t GetHex(uint8_t);
uint8_t atoh(char c);
int ishex(char);
int iswhite(char);

static char hello[] FLASH = {
"\rNewBot monitor\r\
v[l,r]XX\tSet left/right velocity\r\
aXXXX - YYYY\tChange acceleration\r\
K[pdio]XX - YY\tChange PID parameters\r\
g\t\tStart motion\r\
t\t\tTerminate Motion\r\
AXXXX\t- Turn XX degrees\r
T[l,r,b]\t-Turn LEFT/RIGHT/BACK\r
q\t\tresets machine\r\
\r> "
};
static char prompt[] FLASH = {"\r> "};
static char seperator[] FLASH = {" - "};
    
AVRX_TASKDEF(Monitor, 28, 0)
{
    uint16_t TmpLeft = 0, TmpRight = 0, tmp;
    uint8_t *l, *r;

    InitFloorParameters();
    putflashstring(hello);
    while(1)
    {
        switch(getcharecho())
        {
            case 'T':
                switch (getcharecho())
                {
                    case 'l':
                        Turn(LEFT, TRUE);
                        break;
                    case 'r':
                        Turn(RIGHT, TRUE);
                        break;
                    case 'b':
                        Turn(BACK, TRUE);
                        break;
                    default:
                        goto usererror;
                }
                break;
            case 'A':
                tmp = GetHexWord();
                if (inp(SREG) & 0x40)
                    goto usererror;
                putflashstring(seperator);
                PutHexWord(tmp);
                Turn(tmp, FALSE);
                break;
            case 'v':
                switch(getcharecho())
                {
                    case 'l':
                        TmpLeft = GetHex(4);
                        break;
                    case 'r':
                        TmpRight = GetHex(4);
                        break;
                    default:
                        goto usererror;
                }
                if (inp(SREG) & 0x40)
                    goto usererror;
                break;
            case 'g':
                Left.VelocitySetpoint = TmpLeft;
                Right.VelocitySetpoint = TmpRight;
                break;
            case 'a':
                PutHexWord(Left.Acceleration);
                putflashstring(seperator);
                tmp = GetHexWord();
                if (inp(SREG) & 0x40)
                    goto usererror;
                Right.Acceleration = Left.Acceleration = tmp;
                break;
            case 'K':
                switch(getcharecho())
                {
                    case 'p':
                        l = &Left.Kp;
                        r = &Right.Kp;
                        break;
                    case 'd':
                        l = &Left.Kd;
                        r = &Right.Kd;
                        break;
                    case 'i':
                        l = &Left.Ki;
                        r = &Right.Ki;
                        break;
                    case 'o':
                        l = &Left.Ko;
                        r = &Right.Ko;
                        break;
                    default: goto usererror;
                }
                putchar(' ');
                PutHexByte(*l);
                putflashstring(seperator);
                tmp = GetHexByte();
                if (inp(SREG) & 0x40)
                    goto usererror;
                *l = *r = tmp;
                break;
            case 't':
                Left.VelocitySetpoint = 0;
                Right.VelocitySetpoint = 0;
                break;
            case 'q':
                asm( \
                    "cli\n" \
                    "\tclr R30\n" \
                    "\tclr R31\n" \
                    "\tijmp\n" \
                    );
            default:
usererror:
                putchar('?');
        }
        putflashstring(prompt);
    }
}

// Returns a max of count hex digits
// returns T bit to indicate failure...

uint16_t GetHex(uint8_t count)
{
    uint16_t retval = 0;
    uint8_t d;
    outp(0, SREG);  // Clear the T bit
    
    while (count--)
    {
        d = toupper(getcharecho());
        if (ishex(d))
        {
            retval = (retval<<4) + atoh(d);
        }
        else if (iswhite(d))
        {
            break;
        }
        else
        {
            outp(0x40, SREG);
            break;
        }
    }
    return retval;
}

int iswhite(char c)
{
    return (c == ' ') || (c == '\r');
}

int ishex(char c)
{
    switch (c)
    {
        case '0':
        case '1':
        case '2':
        case '3':
        case '4':
        case '5':
        case '6':
        case '7':
        case '8':
        case '9':
        case 'A':
        case 'B':
        case 'C':
        case 'D':
        case 'E':
        case 'F':
            return 1;
        default:
            return 0;
    }
}

uint8_t atoh(char c)
{
    if (c >= 'A')
        return ((c - 'A') + 10);
    else
        return (c - '0');
}


