/*
	serialio.c

*/
#include <io.h>
#include <progmem.h>
#include <interrupt.h>
#include <signal.h>
#include "avrx.h"
#include "serialio.h"

#define FALSE 0
#define TRUE 1
#define UCR_INIT  ((1<<TXEN) | (1<<RXEN) | (1<<RXCIE))

static Mutex RxReady, TxReady;
static uint8_t RxChar, prevchar, pushedflag;

// Initialize the USART hardware with the passed baud rate value.

void InitSerialIO(uint8_t ubrr)
{
    outp(ubrr, UBRR);
    outp(UCR_INIT, UCR);
    sbi(PORTD, PD0);
    pushedflag=FALSE;
}

// Just signal that a charactor has been received.

AVRX_SIGINT(SIG_UART_RECV)
{
    cbi(UCR, RXCIE);                // 10-11 moved out side of prolog
    IntProlog();
    sei();                          // 10-11 added
    RxChar = inp(UDR);
    AvrXIntSetSemaphore(&RxReady);  // 10-11 added "int"
    sbi(UCR, RXCIE);
    Epilog();
}

// Shut down interrupt source, signal waiting task that
// UDR is ready for another charactor.

AVRX_SIGINT(SIG_UART_DATA)
{
    cbi(UCR, UDRIE);
    IntProlog();
    sei();                          // 10-11 added
    AvrXIntSetSemaphore(&TxReady);  // 10-11 added "int"
    Epilog();
}

// Get a charactor from the input stream

uint8_t getchar(void)
{
    if (pushedflag == TRUE)
    {
        pushedflag = FALSE;
        return prevchar;
    }
    AvrXWaitSemaphore(&RxReady);
    return RxChar;
}

// Put a charactor to the output stream

void putchar(uint8_t d)
{
    if ((inp(USR) & BV(UDRE)) == 0)
    {
        sbi(UCR, UDRIE);
        AvrXWaitSemaphore(&TxReady);
    }
    outp(d, UDR);
}

// Get a charactor and echo it as well

uint8_t getcharecho(void)
{
    uint8_t t = getchar();
    putchar(t);
    return t;
}

// Push a charactor into the input buffer,
// returning TRUE if OK, FALSE if full.

uint8_t pushchar(uint8_t d)
{
    prevchar = d;
    pushedflag = TRUE;
    return TRUE;
}

// Need PutString, PutHex, GetHex...

void putflashstring(const char *s)
{
    char d;
    while((d = __lpm_inline((unsigned)s)))
    {
        putchar(d);
        s++;
    }
}
